#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <gdbm.h>
#include <unistd.h>
#include <ftw.h>

#include "sndlib.h"
#include "sfdb.h"

static GDBM_FILE dbf;

static int
process_node (const char *file, const struct stat *sb, int flag)

{
	int fd;
	char *suffix;
	datum key;
	datum content;
	sf_db_record rec;

	if (flag != FTW_F) {
		return 0;
	}

	if ((suffix = strrchr (file, '.')) == NULL) {
		return 0;
	}

	if (*(suffix+1) == '\0') {
		return 0;
	}

	/* We can't realistically check every file - or can we ?
	 */

	if (strcasecmp (suffix+1, "wav") != 0 &&
	    strcasecmp (suffix+1, "aiff") != 0 &&
	    strcasecmp (suffix+1, "snd") != 0 &&
	    strcasecmp (suffix+1, "voc") != 0 &&
	    strcasecmp (suffix+1, "au") != 0 &&
	    strcasecmp (suffix+1, "hcom") != 0 &&
	    strcasecmp (suffix+1, "sf") != 0 &&
	    strcasecmp (suffix+1, "cdr") != 0 &&
	    strcasecmp (suffix+1, "smp") != 0 &&
	    strcasecmp (suffix+1, "maud") != 0 &&
	    strcasecmp (suffix+1, "vwe") != 0) {
		return 0;
	}

	/* OK, it stands a good chance of being a sound file */

	if ((fd = open_sound_input ((char *) file)) < 0) {
		fprintf (stderr, "file \"%s\" could not be opened\n",
			 file);
		return 0;
	}

	rec.channels = sound_chans ((char *) file);
	rec.sample_rate = sound_srate ((char *) file);
	rec.frames = sound_samples ((char *) file) / rec.channels;
	rec.sample_bits = sound_bits_per_sample ((char *) file);
	rec.sample_format = sound_data_format ((char *) file);
	rec.file_type = sound_header_type ((char *) file);

	key.dptr = (char *) file;
	key.dsize = strlen (file) + 1; /* keep null */

	content.dptr = (void *) &rec;
	content.dsize = sizeof (rec);
	
	if (gdbm_store (dbf, key, content, GDBM_REPLACE) != 0) {
		fprintf (stderr, "entry not stored for %s\n", file);
	} else {
		printf ("added: %s\n", file);
	}
	
	close_sound_input (fd);
}

void
add_to_db (char *dbname, char *dirname, int new_db)

{ 
	if (dbname == NULL) {
		fprintf (stderr, "You must specify the database filename\n");
		return;
	}

	dbf = gdbm_open (dbname, 0, 
			 new_db ? GDBM_WRITER|GDBM_NEWDB :
			          GDBM_WRCREAT,
			 0664, NULL);
	
	if (ftw (dirname, process_node, 100) < 0) {
		fprintf (stderr, "directory search of \"%s\" failed\n",
			 dirname);
	}

	gdbm_close (dbf);
}

void
create_db (char *dbname, char *path)

{
	char *dir;
	int dircnt;

	if (dbname == NULL) {
		fprintf (stderr, "You must specify the database filename\n");
		return;
	}

	if (path == NULL) {
		if ((path = getenv ("SFDB_PATH")) == NULL) {
			fprintf (stderr, 
				 "environment variable SFDB_PATH not set\n");
			return;
		} 
	} 


	dir = strtok (path, ":");
	dircnt = 0;

	while (dir) {
		add_to_db (dbname, dir, dircnt == 0);
		dircnt++;
		dir = strtok (NULL, ":");
	}
	
}

void
list_db (char *dbname)

{
	GDBM_FILE dbf;
	datum key;
	datum content;
	sf_db_record *rec;

	if (dbname == NULL) {
		fprintf (stderr, "You must specify the database filename\n");
		return;
	}

	if ((dbf = gdbm_open (dbname, 0, GDBM_READER, 0, NULL)) == 0) {
		fprintf (stderr, "Cannot open database file \"%s\"\n",
			 dbname);
		return;
	}
	
	key = gdbm_firstkey (dbf);
	
	while (key.dptr) {
		int hrs;
		int mins;
		int secs;
		
		content = gdbm_fetch (dbf, key);
		rec = (sf_db_record *) content.dptr;

		secs = (int) ((float) rec->frames / 
			      (float) rec->sample_rate);
		hrs =  secs / 3600;
		secs -= (hrs * 3600);
		mins = secs / 60;
		secs -= (mins * 60);
		
		printf ("%20s %4d %6d %2d %12d %02d:%02d:%02d %8s %8s\n", 
			basename (key.dptr),
			rec->channels,
			rec->sample_rate,
			rec->sample_bits,
			rec->frames,
			hrs, mins, secs,
			sound_type_name (rec->file_type),
			sound_format_name (rec->sample_format));

		key = gdbm_nextkey (dbf, key);
	}

	gdbm_close (dbf);
}
