#include <string.h>
#include <iostream.h>
#include <errno.h>
#include <stdlib.h>
#include <gdbm.h>
#include <unistd.h>
#include <signal.h>

extern "C" {
extern char *basename (const char *); /* why doesn't string.h cause
					 this to be declared ? */
#include "sndlib.h"
}

#include "sfdb_display.h"
#include "sfdb.h"
#include "selector.h"
#include "forkexec.h"

void sfdb_lister (Gtk_CList &list, void *arg);

const size_t sfdbDisplay::Columns;
const gchar *sfdbDisplay::column_titles[sfdbDisplay::Columns] = {
	"File",
	"Type",
	"Format",
	"Rate",
	"Channels",
	"Duration"
};

sfdbDisplay::sfdbDisplay (char *db, bool play_selections)
	: dbname (db),
	  play_on_select (play_selections),
	  selector (sfdb_lister, 
		    dbname, 
		    Columns, true, true,
		    column_titles),
	  stop_play_button ("Stop Play"),
	  exit_button ("Exit"),
	  add_dir_button ("Add Directory to DB"),
	  rescan_button ("Rescan DB")
{
	current_pid = 0;

	connect_to_method (selector.selection_made, 
			   this, &sfdbDisplay::file_selected);
	connect_to_method (stop_play_button.clicked, 
			   this, &sfdbDisplay::stop_play);
	connect_to_method (exit_button.clicked,
			   this, &sfdbDisplay::exit);
	connect_to_method (add_dir_button.clicked, 
			   this, &sfdbDisplay::add_dir);
	connect_to_method (rescan_button.clicked,
			   this, &sfdbDisplay::rescan);

	/* Stupid GTK - using pixel widths breaks this if we 
	   change fonts via an RC file.
	*/

	selector.clist().set_column_width (0, 250);
	selector.clist().set_column_width (1, 50);
	selector.clist().set_column_width (2, 150);
	selector.clist().set_column_width (3, 80);
	/* 4 (channels) can use title width default */
	selector.clist().set_column_width (4, 80);
	selector.clist().set_column_width (5, 100);

	selector.clist().set_sort_column (0);
	
	db_hpacker.set_spacing (5);
	db_hpacker.pack_start (dir_entry);
	db_hpacker.pack_start (add_dir_button, false, false);
	db_hpacker.pack_start (rescan_button, false, false);
	db_vpacker.pack_start (db_hpacker, false, false);

	set_spacing (5);
	set_homogeneous (false);
	pack_start (selector);
	pack_start (rule1, false, false);
	pack_start (db_vpacker, false, false);
	pack_start (rule2, false, false);
	bottom_row.set_spacing (5);

	if (play_on_select) {
		bottom_row.pack_start (stop_play_button, false, false);
	}

	bottom_row.pack_end (exit_button, false, false);
	pack_start (bottom_row, false, false);
	pack_start (rule3, false, false);

	show_all ();
}

sfdbDisplay::~sfdbDisplay ()

{
	stop_play ();
}

void
sfdbDisplay::file_selected (Gtk_Selector *selector, SelectionResult *result)

{
	char cmd[1024];
	extern char **environ;
	char *playcmd;

	stop_play ();

	if (!play_on_select) {
		return;
	}

	if ((playcmd = getenv ("SFDB_PLAYER")) == 0) {
		playcmd = SFDB_PLAYER_DEFAULT;
	}

	sprintf (cmd, "%s %s", playcmd, result->data);

	if ((current_pid = forkexec_cmd (cmd, environ, 0, 0)) < 0) {
		current_pid = 0;
	}
}

void
sfdbDisplay::stop_play ()

{
	if (current_pid) {
		kill (current_pid, SIGKILL);
		current_pid = 0;
	}
}

void
sfdbDisplay::exit ()

{
	stop_play ();
	::exit (0);
}

void
sfdbDisplay::add_dir ()

{
	string dir;

	dir = dir_entry.get_text();

	if (dir.length() == 0) {
		return;
	}

	/* sndlib tries to open /dev/dsp ... sigh */

	stop_play ();

	add_to_db (dbname, (char *) dir.c_str(), 0);
	
	rescan ();
}

void
sfdbDisplay::rescan ()

{
	selector.rescan ();
}

static void
freeOnDestroy (void *arg)

{
	free (arg);
}

void sfdb_lister (Gtk_CList &list, void *arg)

{
	char *dbname = (char *) arg;
	GDBM_FILE dbf;
	datum key;
	datum content;
	const gchar *rowdata[sfdbDisplay::Columns];
	sf_db_record *rec;
	gchar channel_str[16];
	gchar rate_str[16];
	gchar type_str[32];
	gchar format_str[32];
	gchar duration_str[32];
	gchar name_str[1024];
	int rowcnt;

	if ((dbf = gdbm_open (dbname, 0, GDBM_READER, 0, 0)) == 0) {
		return;
	}

	rowdata[0] = name_str;
	rowdata[1] = type_str;
	rowdata[2] = format_str;
	rowdata[3] = rate_str;
	rowdata[4] = channel_str;
	rowdata[5] = duration_str;

	key = gdbm_firstkey (dbf);

	rowcnt = 0;
	while (key.dptr) {
		int hrs;
		int mins;
		int secs;
		int total_secs;
		int frames_remaining;
		float fractional_secs;

		content = gdbm_fetch (dbf, key);
		
		rec = (sf_db_record *) content.dptr;

		secs = (int) ((float) rec->frames / 
			      (float) rec->sample_rate);
		hrs =  secs / 3600;
		secs -= (hrs * 3600);
		mins = secs / 60;
		secs -= (mins * 60);

		total_secs = (hrs * 3600) + (mins * 60) + secs;
		frames_remaining = rec->frames - 
			(total_secs * rec->sample_rate);
		fractional_secs = (float) frames_remaining / rec->sample_rate;

		sprintf (duration_str, "%02d:%02d:%05.2f", 
			 hrs, mins, (float) secs + fractional_secs);
		sprintf (rate_str, "%d", rec->sample_rate);
		if (rec->channels == 1) {
			strcpy (channel_str, "mono");
		} else if (rec->channels == 2) {
			strcpy (channel_str, "stereo");
		} else {
			sprintf (channel_str, "%d", rec->channels);
		}
		strcpy (type_str, sound_type_name (rec->file_type));
		strcpy (format_str, sound_format_name (rec->sample_format));
		strcpy (name_str, basename (key.dptr));

		list.append (rowdata);
		list.set_row_data_full (rowcnt, key.dptr, freeOnDestroy);
		rowcnt++;

		key = gdbm_nextkey (dbf, key);
	}

	list.sort ();

	gdbm_close (dbf);
}

